(function (global, factory) {
	typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports) :
	typeof define === 'function' && define.amd ? define(['exports'], factory) :
	(factory((global.is = {})));
}(this, (function (exports) { 'use strict';

/**
 * Extend or merge an object
 * @param {Object} args The objects to combine
 */


/**
 * Safety measure to ensure an array is always present
 * @param {*} val value to ensure is an array
 */


/**
 * Recursively iterate through an object or array and return the values
 * @param {Object} obj Object to iterate through
 * @param {Function} cb callback to send values back to
 */


/**
 * Formats the results object sent back to the user
 * @param {Object} obj Object to be formatted
 */


/**
 * Validates the sent in schema is useable
 * @param {*} schema The schema value to validate
 */


/**
 * Validates a credit card using the luhn algorithm
 * @param {String} val The card number string to validate
 */
var luhn = function (val) {
  var numArr = [0, 2, 4, 6, 8, 1, 3, 5, 7, 9];
  var stringVal = String(val);
  var len = stringVal.length;
  var bit = 1;
  var sum = 0;
  var num = 0;

  while (len) {
    num = parseInt(stringVal.charAt(--len), 10);
    sum += (bit ^= 1) ? numArr[num] : num; // eslint-disable-line
  }

  return sum && sum % 10 === 0;
};

/* eslint-disable max-len */

var emailRegex = /^[\w\u00c0-\u017f][\w.-_\u00c0-\u017f]*[\w\u00c0-\u017f]+[@][\w\u00c0-\u017f][\w.-_\u00c0-\u017f]*[\w\u00c0-\u017f]+\.[a-z]{2,4}$/i;
var vinRegex = /^[a-hj-npr-z0-9]{9}[a-hj-npr-tv-y1-9]{1}[a-hj-npr-z0-9]{7}$/i;

var isDate = function (val) { return (/^((1[0-2])|(0?[1-9]))[-/.]?((0?[1-9])|([1-2][0-9])|(3[0-1]))[-/.]?(([1-2]{1}[0-9]{3})|([0-9]{2}))$/m).test(val); };

var isDateShort = function (val) { return (/^((1[0-2])|(0?[1-9]))[-/.]?((0?[1-9])|([1-2][0-9])|(3[0-1]))[-/.]?$/m).test(val); };

var isDateProper = function (val) { return (/^(([1-2]{1}[0-9]{3})|([0-9]{2}))[-/.]?((1[0-2])|(0?[1-9]))[-/.]?((0?[1-9])|([1-2][0-9])|(3[0-1]))$/m).test(val); };

var isEmail = function (email) {
  if ( email === void 0 ) email = emailRegex;

  return function (val) {
  if (email.emailPattern) {
    return email.emailPattern.test(val);
  }

  return email.test(val);
};
};

var isNumber = function (val) { return !isNaN(val); };

var isPositive = function (val) { return !isNaN(val) && Number(val) >= 0; };

var isNegative = function (val) { return !isNaN(val) && Number(val) < 0; };

var isVin = function (vin) {
  if ( vin === void 0 ) vin = vinRegex;

  return function (val) {
  if (vin.vinPattern) {
    return vin.vinPattern.test(val);
  }

  return vin.test(val);
};
};

var isZip = function (val) { return (/^\d{5}(-\d{4})?$/).test(val); };

var isCAPostalCode = function (val) { return (/^[ABCEGHJKLMNPRSTVXY]{1}\d{1}[A-Z]{1} *\d{1}[A-Z]{1}\d{1}$/i).test(val); };

var isPhone = function (val) { return (/^[0-9]{10}$/).test(val.replace(/\W/g, '')); };

var isLicensePlate = function (val) { return (/^([A-Z]|[0-9]){1,3}(\s|-|•)?([A-Z]|[0-9]){3,5}$/i).test(val); };

var isVisaCard = function (strict) {
  if ( strict === void 0 ) strict = true;

  return function (val) {
  if (strict) {
    return luhn(val);
  }

  return (/^4[0-9]{15}$/).test(val);
};
};

var isVisaPanCard = function (strict) {
  if ( strict === void 0 ) strict = true;

  return function (val) {
  if (strict) {
    return luhn(val);
  }

  return (/^4[0-9]{18}$/).test(val);
};
};

var isMasterCard = function (strict) {
  if ( strict === void 0 ) strict = true;

  return function (val) {
  if (strict) {
    return luhn(val);
  }

  return (/^5[1-5][0-9]{14}$/).test(val);
};
};

var isAmericanExpressCard = function (strict) {
  if ( strict === void 0 ) strict = true;

  return function (val) {
  if (strict) {
    return luhn(val);
  }

  return (/^3(4|7)[0-9]{13}$/).test(val);
};
};

var isDiscoverCard = function (strict) {
  if ( strict === void 0 ) strict = true;

  return function (val) {
  if (strict) {
    return luhn(val);
  }

  return (/^6[0-9]{15}$/).test(val);
};
};

var isBelowMax = function (m) {
  if ( m === void 0 ) m = Infinity;

  return function (val) {
  if (m.max) {
    return !isNaN(val) && Number(val) < m.max;
  }

  return !isNaN(val) && Number(val) < m;
};
};

var isAboveMin = function (m) {
  if ( m === void 0 ) m = -Infinity;

  return function (val) {
  if (m.min) {
    return !isNaN(val) && Number(val) > m.min;
  }

  return !isNaN(val) && Number(val) > m;
};
};

exports.isDate = isDate;
exports.isDateShort = isDateShort;
exports.isDateProper = isDateProper;
exports.isEmail = isEmail;
exports.isNumber = isNumber;
exports.isPositive = isPositive;
exports.isNegative = isNegative;
exports.isVin = isVin;
exports.isZip = isZip;
exports.isCAPostalCode = isCAPostalCode;
exports.isPhone = isPhone;
exports.isLicensePlate = isLicensePlate;
exports.isVisaCard = isVisaCard;
exports.isVisaPanCard = isVisaPanCard;
exports.isMasterCard = isMasterCard;
exports.isAmericanExpressCard = isAmericanExpressCard;
exports.isDiscoverCard = isDiscoverCard;
exports.isBelowMax = isBelowMax;
exports.isAboveMin = isAboveMin;

Object.defineProperty(exports, '__esModule', { value: true });

})));
