(function (global, factory) {
	typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports) :
	typeof define === 'function' && define.amd ? define(['exports'], factory) :
	(factory((global.meets = {})));
}(this, (function (exports) { 'use strict';

/* eslint-disable max-len */
var passwordRegex = /^(?=.*[a-z])(?=.*[A-Z])(?=.*\d)(?=.*[$@$!%*?&])[A-Za-z\d$@$!%*?&]{8,}$/;

var meetsMinMax = function (ref) {
  var min = ref.min; if ( min === void 0 ) min = -Infinity;
  var max = ref.max; if ( max === void 0 ) max = Infinity;

  return function (val) { return !isNaN(val) && (Number(val) >= min && Number(val) <= max); };
};

var meetsYearStandard = function (val) { return (/(^[0-9]{2}$)|(^[1-2]{1}[0-9]{3}$)/).test(val); };

var meetsCVN = function (val) { return val.length === 3 && (/[0-9]/).test(val); };

var meetsCVNAmex = function (val) { return val.length === 4 && (/[0-9]/).test(val); };

var meetsTreadDepth = function (val) { return (/^(([0-1]?[0-9]|2[0-1])(\.[0-9])?|22)$/i).test(val); };

var meetsPassReq = function (pass) {
  if ( pass === void 0 ) pass = passwordRegex;

  return function (val) {
  if (pass.passwordPattern) {
    return pass.passwordPattern.test(val);
  }

  return pass.test(val);
};
};

exports.meetsMinMax = meetsMinMax;
exports.meetsYearStandard = meetsYearStandard;
exports.meetsCVN = meetsCVN;
exports.meetsCVNAmex = meetsCVNAmex;
exports.meetsTreadDepth = meetsTreadDepth;
exports.meetsPassReq = meetsPassReq;

Object.defineProperty(exports, '__esModule', { value: true });

})));
